% BEM program for results published in Zhang, P., Jung, S., Lee, A., & Xu, Y. (2016). 
% Comparative analysis of nonlinear optofluidic processes in microdroplets. 
% Physical Review E, 93(6), 063119.
% 
% main function to calculate sphere having radiation force and scattering
% force due to WGM in the volume
%
% This version is adapted from version (main_BEM_ax_stokes_WGM_drop.m).
% The main program is divided into sections based on functionality;
% each section is written into functions named in a certain format.
%
% The problem is axisymmetric; BEM formulations are nondimensionalized;
% time steps are non-dimensionalized time;
% Characteristic values are defined same as last version.
% reads data from pre-saved data cases; iterate over all cases;
% data is saved every certain time steps and can be simulated in a separate
% function;
% oil droplet in water; radius of drop varies by case;
%
% version 10/20/15; author: Peng Zhang

%%
close all
clear all
fclose all;
clc

%% global parameters
flag_plot=1;
flag_save=-1;

flag_radiation_p=-1; % flag for radiation pressure: >0: on; <0: off.

flag_save_stream=-1; % flag for velocity field in volume calculations
frame_save_stream=0; % time step at which the velocity is computed
%% WGM in droplet information for all cases

% R and L values for oil droplet in water \lambda=700nm
R_array = [400  300 250 200 150 120 100 80 70 60  50  40
    ];

L_array = [5145  3847  3204  2569  1918  1529  1275    1018  889  761  632  504
    ];

%{
max_vel_array = [
    0.000518137836925
    0.000669162147620
    0.000786952254674
    0.000959162459366
    0.001234992879423
    0.001500598680406
    0.001758846974304
    0.002133109054340
    0.002392467699334
    0.002730097152479
    0.003187854332314
    0.003849085275192];
%}
    
Ncase=length(R_array);

% max_velocity=zeros(Ncase,1);

n_co=1.44; 
cel=3e8; % speed of light
attn=70; % 1/m; approx value
%% simulation parameters
Iflow = 1;  % free space

Igeom=1;

NGL   = 8;
mu    = 1.0;
Ux    = 1.0;
wall  = 0.0;

% for volume velocity computations:
% number of seeding points in r and theta directions:
N_seed_r=32;
N_seed_th=32;
% number of integration points in r and theta directions:
N_int_r=64;
N_int_th=64;

%% begin iteration over all cases
for Icase=7:7%Ncase
    %% stepping parameters
    Ndelt=5e3; %5000%1e4 10
    time=0;
    
    Del_I=25; % number of iterations to save results % 50
    Del_frz=250; % number of iterations to generate new influence matrix
    %     DR_frz=3e-5; %
    %% load parameters
    R_val = R_array(Icase);
    L_val = L_array(Icase);
    M_val = L_val;
    
    file_name = ['WGM_R_' num2str(R_val) ...
        '_L_' num2str(L_val) '_M_' num2str(M_val) '.mat;'];
    file_name2=['WGM_S_field_R_',num2str(R_val),...
        '_L_',num2str(L_val),'_M_',num2str(M_val), '.mat'];

    cmd = ['load ' file_name];  
    cmd2 = ['load ',file_name2];
    
    cd optics_data_oil_in_water
    eval(cmd);
    eval(cmd2);
    cd ..
    
%     lambda=lambda_WGM*(1e-6);
    
    R_mesh=linspace(r_min,r_max,N_r)/R_val;
    Th_mesh=linspace(theta_min,theta_max,N_theta);
    
    [r_mesh,th_mesh]=meshgrid(R_mesh,Th_mesh);
    F_mesh=attn*n_co*(S_mesh/WGM_power)/cel; % in physical unit 
    
    %% characteristic scales:
    Rb=R_val*(1e-6); % m
    gama=0.030; % N/m
    nu=0.838*16e-3; %Pa s
    
    T=nu*Rb/gama; % s
    U=gama/nu; % m/s
    DP=gama/Rb; % characteristic pressure
    
    F_char=gama/(Rb^2); % characteristic force density
    rho_char= nu^2/(gama*Rb);%characteristic mass density %gama*Rb/(nu^2);
    %% nondimensionalization of parameters:
    % geometry
    Rad = 1.0;
    
    % water (cladding) properties : normalized
    nu1=1.0e-3/nu; %
    rho1=1.0e3/rho_char; %
    
    % oil (core) properties
    nu2=1;
    rho2=800/rho_char; %
    
    lamb=nu2/nu1;
    delrho=rho1-rho2;
    
    factor_PL=WGM_power;
    DP0=(optical_pressure_array./factor_PL);%*10 % normalize by max power to get same power for all cases
    
    DP0=[0,DP0,0]./DP; Thp=[0,theta_array,pi]; Xp=cos(Thp);
    
    if flag_radiation_p<0
        DP0=DP0.*0.0;
    end
    
    F_mesh=F_mesh/F_char;
    
    if flag_plot>0
        figure(10)
        [c,h]=contourf(r_mesh,th_mesh,F_mesh,30);
        set(h, 'linestyle', 'none');
        ylabel('\theta');
        xlabel('r');
        title('volume force density');
        colorbar
                
        figure(55)
        plot(Thp,DP0,'-');
        title('radiation pressure');
    end
    %% Problem definition:
    dth=pi/128;
    
    Nelm=128+1;
    
    xcntr = zeros(Nelm,1);
    ycntr = zeros(Nelm,1);
        
    Itype= ones(Nelm,1);
    xel=zeros(Nelm,2);
    yel=zeros(Nelm,2);
    tel=zeros(Nelm,2);
    
    xcl=zeros(Nelm,1);
    ycl=zeros(Nelm,1);
    tcl=zeros(Nelm,1);
    vnex=ones(Nelm+1,1);
    vney=zeros(Nelm+1,1);
    
    %% clear files and save parameters
    Rcase=R_val;
    filenum=num2str(R_val);
    suffx=[filenum '.txt'];
    %
    if flag_save>0
        savefilenames=['infor' suffx ' xcl' suffx ' ycl' suffx ' xel' suffx ...
            ' yel' suffx ' f_sum' suffx ' uv' suffx ' curvt' suffx ' ncxy' suffx...
            ' Re' suffx ' Rc' suffx];
        cmd = ['delete ' savefilenames];  eval(cmd);
        
        dlmwrite(['infor' suffx],[Nelm;Ndelt;Del_I;T;U;DP],'-append', 'delimiter', ' ');
    end
    %}
    
    %% begin iteration over time:
    fprintf(['\n Case R=' filenum ' \n']);
    fprintf('Normalized pressure = %f \n',max(DP0));
    %     fprintf('central pressure (N/m^2) = %f \n',max(DP0));
    fprintf('Characteristic time: %e \n',T);
    fprintf('Characteristic velocity: %f \n',U);
    fprintf('\n');
    
    tic;
        
    for It=0:Ndelt
        
        %% mesh information
        if It==0
            %% initialization
            [ang,angc,x,y,xel,yel,tel,xcl,ycl,tcl,lel,R,Curvt,Itype,ncx,ncy,nex,ney,xcntr,ycntr]=...
                BEM_ax_initial_discrt_WGM_drop(Nelm,Rad,xel,yel,tel,tcl,Itype);
            
        else
            %% mesh update
            time=time+delt;
            
            [x,y,xcl,ycl,xel,yel,tcl,tel,lel,ncx,ncy,nex,ney,xcntr,ycntr,R,Curvt,angle,Itype]=...
                BEM_ax_update_mesh_WGM_drop...
                (Nelm,delt,x,y,u,v,ue,ve,xcl,ycl,xel,yel,tcl,tel,ncx,ncy,Itype);
        end
        
        if flag_plot>0
%             BEM_ax_plot_geometry(Nelm,x,y,xcl,ycl,ncx,ncy,nex,ney);
        end
        
        %% generate linear system
        if mod(It,Del_frz)==0
            [Ls, Uphi]=BEM_ax_sw_stokes_linear_system_WGM_drop...
                (lamb,nu1,Nelm,xcl,ycl,tcl,xel,yel,tel,xcntr,ycntr,R,Itype,NGL,...
                N_int_r,N_int_th,R_val,r_min,r_max,theta_min,theta_max,...
                r_mesh,th_mesh,F_mesh);
        end
        %% solution
        if It==0 % compute magnification factor at first time step
            centfg=0.5*delrho*Uphi.^2;
            magnfc=0.05/max(centfg);
            
            if flag_save>0
                dlmwrite(['infor' suffx],magnfc,'-append', 'delimiter', ' ');
            end
        end

        [u,v,ue,ve,max_u,traction,f_total]=...
            BEM_ax_sw_stokes_solution_WGM_drop...
            (Nelm,Curvt,angc,ang,xcl,ncx,ncy,DP0,Xp,Ls,Uphi,delrho,magnfc);
        
        % plot solutions
        if flag_plot>0
            BEM_ax_plot_solution(Nelm,ang,angc,u,v,ue,ve,f_total,Uphi);
        end
        
        %% compute velocity field in the drop volume
        if flag_save_stream>0
            if frame_save_stream==It
                fprintf('Computing velocity in fluid volume: time=%5.3f. \n',time);
                
                BEM_ax_sw_stokes_volume_velocity_WGM_drop...
                    (Nelm,Iflow,time,u,v,traction,Uphi,nu1,lamb,...
                    angc,xel,yel,tel,xcntr,ycntr,R,Itype,...
                    N_seed_r,N_seed_th,N_int_r,N_int_th,NGL,filenum,...
                    r_mesh,th_mesh,F_mesh,...
                    Rcase,r_min,r_max,theta_min,theta_max,flag_plot);
            end
        end
        
        %% update time step

        if It==0
%             delt=5e-4*(max_vel_array(1)/max_vel_array(Icase)); % time step size
            delt_r=5e-6; % max interface displacement at initial time step
            delt=min(delt_r/max_u,5e-3);
            delt0=delt;
            
            max_u_org=max_u;
        end
        
        if It ==100
            delt0=delt;
            max_u0=max_u; % value of the maximum velocity value
            max_u1=max_u; % value of max vel of last time step
        end
        
        if It>100
            delt=delt0*min([max_u0/max_u,10]);
        end
        
        %% save files and check the termination criteria:
        if mod(It,Del_I)==0
            fprintf('Step: %d, time= %f, time step= %e, max_vel= %e\n', It, time, delt, max_u);
            
            if flag_save>0
                BEM_ax_save_results_WGM_drop...
                    (Nelm,time,xel,yel,xcl,ycl,u,v,ncx,ncy,f_total,Curvt,suffx);
            end
            
            if It>100
                if max_u>max_u1
                    fprintf('Instability occurs. \n');
                    break;
                end
                
                if max_u <= max_u_org/(1e3);
                    fprintf('Solution converges. \n');
                    break;
                end
            end
            
            if max(abs(imag(u)))>1e-20
                fprintf('imaginary solution accurs.\n');
                break;
            end
            
            max_u1=max_u;
        end % if mod(It,Del_I)==0
        
    end % end of iteration of time steps
    %     max_velocity(Icase)=max_u;
    
    if It==Ndelt
        fprintf('Max time steps %d reached. \n',Ndelt);
    end
    
    t=toc;
    hrs=floor(t/3600);
    t=t-hrs*3600;
    mins=floor(t/60);
    t=round(t-mins*60);
    
    fprintf('Computation time for R=%d um: %d hrs, %d mins, %d secs. \n',str2num(filenum),hrs,mins,t);
    
end % end of iteration of R cases
