function [xc,yc,Xct,Yct,R,Curvt,angle,ncx,ncy,nex,ney,TYPE]=circular_seg5(x,y,xc,yc,nx0,ny0,delth)
% function to calculate circular element information based on 3 points in
% an element.
%
% input: 
% x,y: element end points; 
% xc,yc: element center points;
% nx0,ny0: reference normal direction for 1st element
% delth: threshold angle difference for normal vector direction change
% 
% output: 
% xc,yc: updated element center points, with angle value being the mean of edge angles;
% Xct,Yct: coords of center of circular boundary on each element;
% R: radius of circular curve of each element;
% Curvt: curvature of each element;
% 
% Formula for circle formed by three points are found at:
% http://mathforum.org/dr.math/faq/formulas/faq.analygeom_2.html#twocircles
% 
% This version is adapted from circular_seg4, with enforcement of element
% center angle being the mean of edge points angles
% 
% version: 10/10/14

Nelm=length(xc);

if (length(x)-Nelm)~=1
    disp('Length of vectors do not match!/n');
    return;
end

R=zeros(Nelm,1);
Curvt=zeros(Nelm,1);
Xct=zeros(Nelm,1);
Yct=zeros(Nelm,1);
angle=zeros(Nelm,3);
ncx=zeros(Nelm,1);
ncy=zeros(Nelm,1);
nex=zeros(Nelm,2);
ney=zeros(Nelm,2);
TYPE=zeros(Nelm,1); % 1:for straight elements; 2:for circular elements.

for Ielm=1:Nelm
    
    x1=x(Ielm); x2=xc(Ielm); x3=x(Ielm+1);
    y1=y(Ielm); y2=yc(Ielm); y3=y(Ielm+1);
    
    matA=[x1,y1,1;x2,y2,1;x3,y3,1]; A=det(matA);
    matD=[x1^2+y1^2,y1,1;x2^2+y2^2,y2,1;x3^2+y3^2,y3,1];
    D=-det(matD);
    matE=[x1^2+y1^2,x1,1;x2^2+y2^2,x2,1;x3^2+y3^2,x3,1];
    E=det(matE);
    matE=[x1^2+y1^2,x1,y1;x2^2+y2^2,x2,y2;x3^2+y3^2,x3,y3];
    F=-det(matE);
    
    temp1=((x2-x1)*(y3-y1)-(y2-y1)*(x3-x1));
    if temp1>1e-15
        alpha=1;
    elseif temp1<-1e-15
        alpha=-1;
    else
        alpha=0;
        Curvt(Ielm)=0;
        Xct(Ielm)=Inf;
        Yct(Ielm)=Inf;
        R(Ielm)=Inf;
        TYPE(Ielm)=1;
        
        %         if abs(D)<=1e-10
        %             angle(Ielm,:)=-pi/2;
        %         elseif abs(E)<=1e-10
        %             angle(Ielm,:)=-pi;
        %         else
        %             angle(Ielm,:)=pi-abs(atan(D/E)); % k=-D/E,-1/k=E/D,
        %         end
        
        th=atan(E/D);
        angle(Ielm,:)=sign(th)*(abs(th)-pi);
        ncx(Ielm)=sqrt(D^2/(D^2+E^2));
        ncy(Ielm)=E/D*ncx(Ielm);
        
        nex(Ielm,:)=ncx(Ielm);
        ney(Ielm,:)=ncy(Ielm);
        
        % enforce the element center to be the center point on the segment
        xc(Ielm)=0.5*(x1+x3);
        yc(Ielm)=0.5*(y1+y3);
        
    end
    
    if abs(alpha)>0.5
        Xct(Ielm)=-D/(2*A);
        Yct(Ielm)=-E/(2*A);
        R(Ielm)=sqrt(D^2+E^2-4*A*F)/(2*abs(A));
        TYPE(Ielm)=2;
        
        % calculate curvature:
        sintheta=(y2-Yct(Ielm))/R(Ielm);
        Curvt(Ielm)=alpha/2*(1/R(Ielm)+sintheta/(Yct(Ielm)+R(Ielm)*sintheta));
        
        % calculate angles at 'mid' and end points:
        angle(Ielm,1)=sign(y1-Yct(Ielm))*real(acos((x1-Xct(Ielm))/R(Ielm)));
        th=angle(Ielm,1);
        nex(Ielm,1)=cos(th); % sign(sin(th))*
        ney(Ielm,1)=sin(th);
        
        temp=(x2-Xct(Ielm))*(x1-Xct(Ielm))+(y2-Yct(Ielm))*(y1-Yct(Ielm));
        angle(Ielm,2)=angle(Ielm,1)+alpha*real(acos(temp/(R(Ielm))^2));
        th=angle(Ielm,2);
        ncx(Ielm)=cos(th);
        ncy(Ielm)=sin(th);
        
        temp=(x2-Xct(Ielm))*(x3-Xct(Ielm))+(y2-Yct(Ielm))*(y3-Yct(Ielm));
        angle(Ielm,3)=angle(Ielm,2)+alpha*real(acos(temp/(R(Ielm))^2));
        th=angle(Ielm,3);
        nex(Ielm,2)=cos(th);
        ney(Ielm,2)=sin(th);
        
        % re-define mid point having mean angle
        if R(Ielm)<0.01*1e4
            new_angc=0.5*(angle(Ielm,1)+angle(Ielm,3));
            angle(Ielm,2)=new_angc;
            xc(Ielm)=Xct(Ielm)+R(Ielm)*cos(new_angc);
            yc(Ielm)=Yct(Ielm)+R(Ielm)*sin(new_angc);
        end
    end
    %{
    if abs(x2)<=0.05
        nex(Ielm,:)=nex(Ielm,:).*sign(ney(Ielm,:));
        ney(Ielm,:)=ney(Ielm,:).*sign(ney(Ielm,:));
        ncx(Ielm)=ncx(Ielm)*sign(ncy(Ielm));
        ncy(Ielm)=ncy(Ielm)*sign(ncy(Ielm));
    end
    %}
    
    if Ielm>=2
        df=sqrt((nex(Ielm,1)-nex(Ielm-1,2)).^2+(ney(Ielm,1)-ney(Ielm-1,2)).^2);
        if df>delth
            nex(Ielm,1)=nex(Ielm,1)*(-1);
            ney(Ielm,1)=ney(Ielm,1)*(-1);
        end
        
%         df=sqrt((ncx(Ielm)-ncx(Ielm-1)).^2+(ncy(Ielm)-ncy(Ielm-1)).^2);
        df=sqrt((ncx(Ielm)-nex(Ielm,1)).^2+(ncy(Ielm)-ney(Ielm,1)).^2);
        if df>delth
            ncx(Ielm)=ncx(Ielm)*(-1);
            ncy(Ielm)=ncy(Ielm)*(-1);
        end
        
%         df=sqrt((nex(Ielm,2)-nex(Ielm-1,2)).^2+(ney(Ielm,2)-ney(Ielm-1,2)).^2);
        df=sqrt((nex(Ielm,2)-ncx(Ielm)).^2+(ney(Ielm,2)-ncy(Ielm)).^2);
        if df>delth
            nex(Ielm,2)=nex(Ielm,2)*(-1);
            ney(Ielm,2)=ney(Ielm,2)*(-1);
        end
        

        
    else
        df=sqrt((nex(Ielm,1)-nx0).^2+(ney(Ielm,1)-ny0).^2);
        if df>delth
            nex(Ielm,1)=nex(Ielm,1)*(-1);
            ney(Ielm,1)=ney(Ielm,1)*(-1);
        end
        
        df=sqrt((ncx(Ielm)-nex(Ielm,1)).^2+(ncy(Ielm)-ney(Ielm,1)).^2);
        if df>delth
            ncx(Ielm)=ncx(Ielm)*(-1);
            ncy(Ielm)=ncy(Ielm)*(-1);
        end
        
        df=sqrt((nex(Ielm,2)-ncx(Ielm)).^2+(ney(Ielm,2)-ncy(Ielm)).^2);
        if df>delth
            nex(Ielm,2)=nex(Ielm,2)*(-1);
            ney(Ielm,2)=ney(Ielm,2)*(-1);
        end
        

        
    end
    
    
end