% BEM program for results published in Zhang, P., Jung, S., Lee, A., & Xu, Y. (2016). 
% Comparative analysis of nonlinear optofluidic processes in microdroplets. 
% Physical Review E, 93(6), 063119.
% 
% main function to calculate spherical shaped ball having a ring of force
% on the equator line;
%
% This version is adapted from 3nd version (prtcl_ax_drop_case3.m).
% The main program is divided into sections based on functionality;
% each section is written into functions named in a certain format.
%
% The problem is axisymmetric; BEM formulations are nondimensionalized;
% time steps are non-dimensionalized time;
% Characteristic values are defined same as 1st - 3rd version.
% reads data from pre-saved data cases; iterate over all cases;
% data is saved every certain time steps and can be simulated in a separate
% function;
% oil droplet in water; radius of drop varies by case;
%
% version 11/29/15; author: Peng Zhang

%%
close all
clear all
fclose all;
clc

%% global parameters
flag_plot=1;
flag_save=-1;

flag_radiation_p=-1; % flag for radiation pressure: >0: on; <0: off.

flag_save_stream=-1; % flag for velocity field in volume calculations
frame_save_stream=0; % time step at which the velocity is computed
%% WGM in droplet information for all cases

% R and L values for oil droplet in water \lambda=700nm
R_array = [400  300 250 200 150 120 100 80 70 60  50  40
    ];

L_array = [5145  3847  3204  2569  1918  1529  1275    1018  889  761  632  504
    ];

Ncase=length(R_array);

%% simulation parameters
Iflow = 1;  % free space

Igeom=1;

NGL   = 8;
mu    = 1.0;
Ux    = 1.0;
wall  = 0.0;

% for volume velocity computations:
Nr=32; % number of seeding points in r and theta directions
Ntheta=32;

%% begin iteration over all cases

for Icase=7:7%Ncase
    %% stepping parameters
    Ndelt=1e4; %5000 %1e4 %10
    time=0;
    
    Del_I=25; % number of iterations to save results % 50
    Del_frz=200; % number of iterations to generate new influence matrix
    %     DR_frz=3e-5; %
    %% load parameters
    R_val = R_array(Icase);
    L_val = L_array(Icase);
    M_val = L_val;
    
    file_name = ['WGM_R_' num2str(R_val) ...
        '_L_' num2str(L_val) '_M_' num2str(M_val) '.mat;'];
    
    cmd = ['load ' file_name];  
    
    cd optics_data_oil_in_water
    eval(cmd);
    cd ..
    
    lambda=lambda_WGM*(1e-6);
    
    %% characteristic scales:
    Rb=R_val*(1e-6); % m
    gama=0.030; % N/m
    nu=0.838*16e-3; %Pa s
    
    T=nu*Rb/gama; % s
    U=gama/nu; % m/s
    DP=gama/Rb; % characteristic pressure
    
    %% nondimensionalization of parameters:
    % geometry
    Rad = 1.0;
    
    % water (cladding) properties : normalized
    nu1=1.0e-3/nu; %
    
    % oil (core) properties
    nu2=1;
    
    lamb=nu2/nu1;
    
    %% radiation pressure (normal pressure)
    factor_PL=WGM_power;
    DP0=(optical_pressure_array./factor_PL);%*10 % normalize by max power to get same power for all cases
    
    DPn=[0,DP0,0]./DP; 
    ThPn=[0,theta_array,pi]; 
    XPn=cos(ThPn);
    
    if flag_radiation_p<0
        DPn=DPn.*0.0;
    end
    %% thermocapillary pressure (shear pressure)
    [ThPt,DPt]=func_thermocapillary_stress(R_val,gama);
    DPt=DPt/DP;
    DPt=[0,DPt,0]; 
    ThPt=[0,ThPt,pi]; 
    XPt=cos(ThPt);
    
    if flag_plot>0
        figure(55)
        clf;
        hold on;
        plot(ThPn,DPn,'r-');
        plot(ThPt,DPt,'b-');
        title('interface force');
        legend('Radiation','Thermocap');
    end
    %% Problem definition:
    dth=pi/128;
    
    Nelm=128+1;
    
    xcntr = zeros(Nelm,1);
    ycntr = zeros(Nelm,1);
    
    tilt  = 0.0;
    
    Itype= ones(Nelm,1);
    xel=zeros(Nelm,2);
    yel=zeros(Nelm,2);
    tel=zeros(Nelm,2);
    
    xcl=zeros(Nelm,1);
    ycl=zeros(Nelm,1);
    tcl=zeros(Nelm,1);
    vnex=ones(Nelm+1,1);
    vney=zeros(Nelm+1,1);
    
    %% clear files and save parameters
    filenum=num2str(R_val);
    suffx=[filenum '.txt'];
    %
    if flag_save>0
        savefilenames=['infor' suffx ' xcl' suffx ' ycl' suffx ' xel' suffx ...
            ' yel' suffx ' fn' suffx ' uv' suffx ' curvt' suffx ' ncxy' suffx...
            ' Re' suffx ' Rc' suffx];
        cmd = ['delete ' savefilenames];  eval(cmd);
        
        dlmwrite(['infor' suffx],[Nelm;Ndelt;Del_I;T;U;DP],'-append', 'delimiter', ' ');
    end
    %}
    
    %% begin iteration over time:
    fprintf(['\n Case R=' filenum ' \n']);
    fprintf('Normalized pressure = %f \n',max(DP0));
    %     fprintf('central pressure (N/m^2) = %f \n',max(DP0));
    fprintf('Characteristic time: %e \n',T);
    fprintf('Characteristic velocity: %f \n',U);
    fprintf('\n');
    
    tic;
    
    %     DR=10.0;
    
    for It=0:Ndelt
        
        %% mesh information
        if It==0
            %% initialization
            [ang,angc,x,y,xel,yel,tel,xcl,ycl,tcl,lel,R,Curvt,Itype,ncx,ncy,nex,ney,xcntr,ycntr]=...
                BEM_ax_initial_discrt_WGM_drop(Nelm,Rad,xel,yel,tel,tcl,Itype); 
            
            if flag_plot>0
                BEM_ax_plot_geometry(Nelm,x,y,xcl,ycl,ncx,ncy,nex,ney);
            end
        else
            %% mesh update
            time=time+delt; 
            
            [x,y,xcl,ycl,xel,yel,tcl,tel,lel,ncx,ncy,nex,ney,xcntr,ycntr,R,Curvt,angle,Itype]=...
                BEM_ax_update_mesh_WGM_drop...
                (Nelm,delt,x,y,u,v,ue,ve,xcl,ycl,xel,yel,tcl,tel,ncx,ncy,nex,ney,Itype); 
        end
                
        %% generate linear system
        if mod(It,Del_frz)==0
            %         if DR>DR_frz
            %             fprintf('Matrix updated; DR=%f; Step=%d \n',DR,It);
            
            Ls=BEM_ax_stokes_linear_system_WGM_drop...
                (lamb,nu1,Nelm,xcl,ycl,tcl,xel,yel,tel,xcntr,ycntr,R,Itype,Iflow,NGL);
            
            %             DR=0.0; % reset DR after matrix update
        end
        %% solution
        [u,v,ue,ve,max_u,traction,fn_total,ft_total]=...
            BEM_ax_stokes_solution_WGM_drop...
            (Nelm,Curvt,angc,ang,xcl,ncx,ncy,DPn,XPn,DPt,XPt,Ls);
        
        % plot solutions
        if flag_plot>0
            BEM_ax_plot_solution(Nelm,ang,angc,u,v,ue,ve,xcl,ycl,ncx,ncy,traction,fn_total,ft_total);
        end
        
        %% compute velocity field in the drop volume
        if flag_save_stream>0
            if frame_save_stream==It
                fprintf('Computing velocity in fluid volume: time=%5.3f. \n',time);
                
                BEM_ax_stokes_volume_velocity_WGM_drop...
                    (Nelm,Iflow,time,u,v,traction,nu1,lamb,angc,xel,yel,tel,...
                    xcntr,ycntr,R,Itype,Nr,Ntheta,NGL,filenum,flag_plot);
            end
        end
        
        %% update time step
        if It==0
            delt_r=7e-7; % max interface displacement at initial time step
            delt=min(delt_r/max_u,1e-3);
            delt0=delt;
    
            max_u_org=max_u;
        end
        
        if It ==100
            delt0=delt;
            max_u0=max_u; % value of the maximum velocity value
            max_u1=max_u; % value of max vel of last time step
        end
        
        if It>100
            delt=delt0*min([sqrt(max_u0/max_u),5]);
        end
        
        %         DR=DR+delt*max_u;
        %% save files and check the termination criteria:
        if mod(It,Del_I)==0
            fprintf('Step: %d, time= %f, time step= %e, max_vel= %e\n', It, time, delt, max_u);
            
            if flag_save>0
                BEM_ax_save_results_WGM_drop...
                    (Nelm,time,xel,yel,xcl,ycl,u,v,ncx,ncy,fn_total,ft_total,Curvt,suffx);
            end
            
            if It>100
                if max_u>max_u1
                    fprintf('Instability occurs. \n');
                    break;
                end
                
                if max_u <= max_u_org/(5e2);
                    fprintf('Solution converges. \n');
                    break;
                end
            end
            
            if max(abs(imag(u)))>1e-20
                fprintf('imaginary solution accurs.\n');
                break;
            end
            
        end % if mod(It,Del_I)==0
        max_u1=max_u;
    end % end of iteration of time steps

    if flag_save_stream>0 
            fprintf('Computing velocity in fluid volume: time=%5.3f. \n',time);
            
            BEM_ax_stokes_volume_velocity_WGM_drop...
                (Nelm,Iflow,time,u,v,traction,nu1,lamb,angc,xel,yel,tel,...
                xcntr,ycntr,R,Itype,Nr,Ntheta,NGL,filenum,flag_plot);
    end
    
    if It==Ndelt
        fprintf('Max time steps %d reached. \n',Ndelt);
    end
    
    t=toc;
    hrs=floor(t/3600);
    t=t-hrs*3600;
    mins=floor(t/60);
    t=round(t-mins*60);
    
    fprintf('Computation time for R=%d um: %d hrs, %d mins, %d secs. \n',str2num(filenum),hrs,mins,t);
    
end % end of iteration of R cases
