function [Qxx,Qxy,Qyx,Qyy,Dxx,Dxy,Dyx,Dyy] = prtcl_ax_slp_2 ...
    ...
    (Iflow ...
    ,X0,Y0,T0 ...
    ,X1,Y1,T1 ...
    ,X2,Y2,T2 ...
    ,NGL ...
    ,Ising ...
    ,Itype ...
    ,xcntr,ycntr ...
    ,amaj,amin,tilt ...
    ,wall ...
    )

%-----------------------------------------
% FDLIB, BEMLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved.
%
% This program is to be used only under the
% stipulations of the licensing agreement.
%----------------------------------------

%-----------------------------------------------------
% Computation of the single-layer potential at
% collocation points located at the mid-point of
% a straight element or a native segment of an ellipse
%
% SYMBOLS:
% -------
%
% Qij:	components of the slp
%-----------------------------------------------------

%   Itype
%   xcntr,ycntr
%   amaj,amin,tilt
%   T1,T2,T0
%   X1,X2
%  Y1,Y2
%  pause

%-----------
% quadrature
%-----------

[ZZ,WW] = gauss_leg(NGL);

%--------
% prepare
%--------

Iopt = 2;

%-----------
% initialize
%-----------

Qxx = 0.0;
Qxy = 0.0;
Qyx = 0.0;
Qyy = 0.0;

Dxx = 0.0;
Dxy = 0.0;
Dyx = 0.0;
Dyy = 0.0;

%---
% prepare for the quadrture
%---

if(Itype==1) % straight segments
    
    XM = 0.5D0*(X2+X1);
    XD = 0.5D0*(X2-X1);
    YM = 0.5D0*(Y2+Y1);
    YD = 0.5D0*(Y2-Y1);
    DR = sqrt(XD*XD+YD*YD);
    vnx =  YD/DR;    % unit normal vector
    vny = -XD/DR;    % points into the flow
    fcc = DR;        % factor for numerical integration
    
else       % elliptical elements
    
    cs = cos(tilt);
    sn = sin(tilt);
    TM = 0.5D0*(T2+T1);
    TD = 0.5D0*(T2-T1);
    ornt = 1.0;
    if(TD<0) ornt = -1.0; end
    
    css  = cos(T0);     % for desingularization
    snn  = sin(T0);
    tmpx = amin*css;
    tmpy = amaj*snn;
    alm0 = sqrt(tmpx*tmpx+tmpy*tmpy); % arc length metric
    DR   = alm0*abs(TD);
    
end

%---
% loop over Gaussian points
%---

for i=1:NGL
    
    if(Itype==1)
        X = XM + XD*ZZ(i);
        Y = YM + YD*ZZ(i);
    else
        T    = TM + TD*ZZ(i);
        css  = cos(T);
        snn  = sin(T);
        tmpx = amaj*css;
        tmpy = amin*snn;
        X    = xcntr + tmpx*cs-tmpy*sn;
        Y    = ycntr + tmpx*sn+tmpy*cs;
        tmpx = amin*css;
        tmpy = amaj*snn;
        alm  = sqrt(tmpx*tmpx+tmpy*tmpy);   % arc length
        tmpx = tmpx/alm;
        tmpy = tmpy/alm;
        vnx  = tmpx*cs-tmpy*sn;
        vny  = tmpx*sn+tmpy*cs;
        vnx  = vnx * ornt;   % unit normal vector
        vny  = vny * ornt;   % points into the flow
        fcc  = alm*TD;        % factor for numerical integration
    end
    
    %------------------
    if(Iflow==1)   % flow in free space
        %-----------------
        
        [Gxx,Gxy ...
            ,Gyx,Gyy ...
            ,QXXX,QXXY,QXYX,QXYY ...
            ,QYXX,QYXY,QYYX,QYYY ...
            ,PXX,PXY,PYX,PYY ...
            ,Iaxis] ...
            ...
            = sgf_ax_fs (Iopt,X,Y,X0,Y0);
        
        %------------------
    elseif(Iflow==2)   % flow bounded by a wall at x=wall
        %-----------------
        
        [Gxx,Gxy ...
            ,Gyx,Gyy ...
            ,QXXX,QXXY,QXYX,QXYY ...
            ,QYXX,QYXY,QYYX,QYYY ...
            ,PXX,PXY,PYX,PYY ...
            ,Iaxis] ...
            ...
            = sgf_ax_w (Iopt,X,Y,X0,Y0,wall);
        
        %------------
    end
    %------------
    
    %-------
    % subtract off the singularity
    %-------
    
    if(Ising==1)
        
        % single layer integral:
        if(Itype==1)
            Dists = (X-X0)^2+(Y-Y0)^2;
            DD    = 0.5D0*log(Dists);
        elseif(Itype==2)
            Dists = ( alm0*(T0-T) )^2;
            DD    = 0.5D0*alm0/alm * log(Dists);
        end
        
        Gxx = Gxx + 2.0D0*DD;
        Gyy = Gyy + 2.0D0*DD;
        
        
    end
    
    %----------
    % end of subtract off the singularity
    %----------
    
    WI = WW(i)*fcc;
    
    Qxx = Qxx + Gxx * WI;
    Qxy = Qxy + Gxy * WI;
    Qyx = Qyx + Gyx * WI;
    Qyy = Qyy + Gyy * WI;
    
    Dxx = Dxx + (QXXX * vnx + QXXY * vny)*WI;
    Dxy = Dxy + (QXYX * vnx + QXYY * vny)*WI;
    
    Dyx = Dyx + (QYXX * vnx + QYXY * vny)*WI;
    Dyy = Dyy + (QYYX * vnx + QYYY * vny)*WI;
    
    %---
end % of Gaussian quadrature
%---

%--------------------------------
% add singularity back to the slp
%--------------------------------

if(Ising==1)
    
    deduction = 4.0D0*DR*(log(DR)-1.0D0);
    Qxx = Qxx - deduction;
    Qyy = Qyy - deduction;
    
end

%-----
% done
%-----

return
